@@ -1,7 +1,6 @@
 //! A client connection to BigML.
 
 use bytes::Bytes;
-use failure::Fail;
 use futures::{prelude::*, FutureExt};
 use reqwest::{self, multipart, StatusCode};
 use serde::de::DeserializeOwned;
@@ -14,6 +13,8 @@ use std::sync::{Arc, RwLock};
 use std::time::Duration;
 use tokio::fs;
 use tokio_util::codec;
+use tracing::debug;
+use tracing::instrument;
 use url::Url;
 
 use crate::errors::*;
@@ -45,6 +46,7 @@ impl Client {
 
     /// Create a new `Client`, specifying the BigML domain to connect to. Use
     /// this if you have a specially hosted BigML instance.
+    #[instrument(level = "trace", skip(username, api_key))]
     pub fn new_with_domain<S1, S2>(
         domain: &str,
         username: S1,
@@ -73,9 +75,9 @@ impl Client {
         let domain = env::var("BIGML_DOMAIN")
             .unwrap_or_else(|_| DEFAULT_BIGML_DOMAIN.to_owned());
         let username = env::var("BIGML_USERNAME")
-            .map_err(|_| format_err!("must specify BIGML_USERNAME"))?;
+            .map_err(|_| Error::missing_env_var("BIGML_USERNAME"))?;
         let api_key = env::var("BIGML_API_KEY")
-            .map_err(|_| format_err!("must specify BIGML_API_KEY"))?;
+            .map_err(|_| Error::missing_env_var("BIGML_API_KEY"))?;
         Self::new_with_domain(&domain, username, api_key)
     }
 
@@ -93,6 +95,7 @@ impl Client {
     }
 
     /// Create a new resource.
+    #[instrument(level = "trace", skip(self, args))]
     pub async fn create<'a, Args>(&'a self, args: &'a Args) -> Result<Args::Resource>
     where
         Args: resource::Args,
@@ -114,6 +117,7 @@ impl Client {
     }
 
     /// Create a new resource, and wait until it is ready.
+    #[instrument(level = "trace", skip(self, args))]
     pub async fn create_and_wait<'a, Args>(
         &'a self,
         args: &'a Args,
@@ -170,7 +174,7 @@ impl Client {
         let err_path = path.clone();
         let stream = codec::FramedRead::new(file, codec::BytesCodec::new())
             .map_ok(|bytes| bytes.freeze())
-            .map_err(move |err| Error::could_not_read_file(&err_path, err).compat());
+            .map_err(move |err| Error::could_not_read_file(&err_path, err));
 
         // Create our source.
         let filename = path.to_string_lossy();
@@ -197,6 +201,7 @@ impl Client {
     /// Update the specified `resource` using `update`. We do not return the
     /// updated resource because of peculiarities with BigML's API, but you
     /// can always use `Client::fetch` if you need the updated version.
+    #[instrument(level = "trace", skip(self))]
     pub async fn update<'a, R: Resource + Updatable>(
         &'a self,
         resource: &'a Id<R>,
@@ -221,6 +226,7 @@ impl Client {
     }
 
     /// Fetch an existing resource.
+    #[instrument(level = "trace", skip(self))]
     pub async fn fetch<'a, R: Resource>(&'a self, resource: &'a Id<R>) -> Result<R> {
         let url = self.url(resource.as_str());
         let client = reqwest::Client::new();
@@ -236,6 +242,7 @@ impl Client {
     ///
     /// If an underlying BigML error occurs, it can be accessed using
     /// [`Error::original_bigml_error`].
+    #[instrument(level = "trace", skip(self))]
     pub async fn wait<'a, R: Resource>(&'a self, resource: &'a Id<R>) -> Result<R> {
         let options = WaitOptions::default()
             .backoff_type(BackoffType::Exponential)
@@ -251,6 +258,7 @@ impl Client {
     ///
     /// If an underlying BigML error occurs, it can be accessed using
     /// [`Error::original_bigml_error`].
+    #[instrument(level = "trace", skip(self, wait_options, progress_options))]
     pub async fn wait_opt<'a, 'b, R: Resource>(
         &self,
         resource: &'a Id<R>,
@@ -328,6 +336,7 @@ impl Client {
 
     /// Download a resource as a CSV file.  This only makes sense for
     /// certain kinds of resources.
+    #[instrument(level = "trace", skip(self))]
     pub async fn download_opt<'a, R: Resource>(
         &'a self,
         resource: &'a Id<R>,
@@ -376,6 +385,7 @@ impl Client {
     }
 
     /// Delete the specified resource.
+    #[instrument(level = "trace", skip(self))]
     pub async fn delete<'a, R: Resource>(&'a self, resource: &'a Id<R>) -> Result<()> {
         let url = self.url(resource.as_str());
         let client = reqwest::Client::new();
@@ -394,6 +404,7 @@ impl Client {
 
     /// Handle a response from the server, deserializing it as the
     /// appropriate type.
+    #[instrument(level = "trace", skip(self, url, res))]
     async fn handle_response_and_deserialize<'a, T>(
         &'a self,
         url: &'a Url,
@@ -435,8 +446,10 @@ impl Client {
 #[test]
 fn client_url_is_sanitizable() {
     let client = Client::new("example", "secret").unwrap();
-    let err: Error =
-        Error::could_not_access_url(&client.url("/test"), format_err!("Details"));
+    let err_msg = Error::Other {
+        source: "Details".into(),
+    };
+    let err = Error::could_not_access_url(&client.url("/test"), err_msg);
     let err_str = format!("{}", err);
     println!("err_str = {:?}", err_str);
     assert!(!err_str.contains("secret"));
